from pyramid.config import Configurator
import os
import warnings
from dogpile.cache import make_region

from crabpy.gateway.capakey import CapakeyGateway
from crabpy.gateway.crab import CrabGateway
from crabpy.client import capakey_factory, crab_factory
from zope.interface import Interface

from crabpy_pyramid.utils import (
    json_list_renderer,
    json_item_renderer
)


class ICapakey(Interface):
    pass


class ICrab(Interface):
    pass

def _parse_settings(settings):
    args = {}
    defaults = {
        'user': None,
        'password': None
    }
    args = defaults.copy()

    # set settings
    for short_key_name in ('user', 'password'):
        key_name = "capakey.%s" % (short_key_name)
        if key_name in settings:
            args[short_key_name] = settings.get(
                key_name, defaults.get(short_key_name)
            )
    # not set user or password 
    for short_key_name in ('user', 'password'):
        if (
            (not short_key_name in args) or
            args[short_key_name] is None
        ):
            warnings.warn(
                '%s was not found in the settings, \
    capakey needs this parameter to function properly.' % short_key_name,
                UserWarning
            )
    return args


def _set_caches(settings, gateway, c):
    #create cache
    root = settings.get('root')
    if not os.path.exists(root):
        os.makedirs(root)
    for name in ('permanent', 'long', 'short'):
        gateway.caches[name] = make_region(key_mangler=str)
        gateway.caches[name].configure_from_config(settings, '%s.%s.' % (c, name))


def _build_capakey(registry):
    capakey = registry.queryUtility(ICapakey)
    if capakey is not None:
        return capakey
    settings = registry.settings
    capakey_settings = _parse_settings(settings)
    factory = capakey_factory(**capakey_settings)
    gateway = CapakeyGateway(factory)
    _set_caches(settings, gateway, 'capakey')

    registry.registerUtility(gateway, ICapakey)
    return registry.queryUtility(ICapakey)


def _build_crab(registry):
    crab = registry.queryUtility(ICrab)
    if crab is not None:
        return crab
    settings = registry.settings
    factory = crab_factory()
    gateway = CrabGateway(factory)
    _set_caches(settings, gateway, 'crab')
    
    registry.registerUtility(gateway, ICrab)
    return registry.queryUtility(ICapakey)


def get_capakey(registry):
    '''
    Get the Capakey Gateway
    '''
    #argument might be a config or a request
    regis = getattr(registry, 'registry', None)
    if regis is None:
        regis = registry

    return regis.queryUtility(ICapakey)
    
def get_crab(registry):
    '''
    Get the Crab Gateway
    '''
    #argument might be a config or a request
    regis = getattr(registry, 'registry', None)
    if regis is None:
        regis = registry
        
    return regis.queryUtility(ICrab)


def includeme(config):
    
    # including routes
    config.include('crabpy_pyramid.routes')
    
    config.add_renderer('listjson', json_list_renderer)
    config.add_renderer('itemjson', json_item_renderer)
    _build_capakey(config.registry)
    _build_crab(config.registry)
    config.add_directive('get_capakey', get_capakey)
    config.add_directive('get_crab', get_crab)
    config.add_request_method(get_capakey, 'capakey_gateway')
    config.add_request_method(get_crab, 'crab_gateway')


def main(global_config, **settings):
    '''
     This function returns a Pyramid WSGI application.
    '''
    config = Configurator(settings=settings)


    
    includeme(config)
    config.scan()
    return config.make_wsgi_app()



from pyramid.view import view_config
from .utils import range_return

''' Views for capakey services '''

@view_config(route_name='list_gemeenten', renderer='listjson', accept='application/json')
def list_gemeenten(request):
    Gateway = request.capakey_gateway()
    gemeenten = Gateway.list_gemeenten(1)
    r = range_return(request)
    return gemeenten[r[0]: r[1]]


@view_config(route_name='get_gemeente', renderer='itemjson', accept='application/json')
def get_gemeente_by_niscode(request):
    Gateway = request.capakey_gateway()
    gemeente_id = int(request.matchdict.get('gemeente_id'))
    return Gateway.get_gemeente_by_id(gemeente_id)


@view_config(
    route_name='list_kadastrale_afdelingen_by_gemeente',
    renderer='listjson', accept='application/json'
)
def list_kadastrale_afdelingen_by_gemeente(request):
    Gateway = request.capakey_gateway()
    gemeente_id = request.matchdict.get('gemeente_id')
    afdelingen = Gateway.list_kadastrale_afdelingen_by_gemeente(gemeente_id)
    r = range_return(request)
    return afdelingen[r[0]:r[1]]


@view_config(route_name='list_kadastrale_afdelingen', renderer='listjson', accept='application/json')
def list_kadastrale_afdelingen(request):
    Gateway = request.capakey_gateway()
    afdelingen = Gateway.list_kadastrale_afdelingen()
    r = range_return(request)
    return afdelingen[r[0]:r[1]]


@view_config(
    route_name='get_kadastrale_afdeling_by_id',
    renderer='itemjson', accept='application/json'
)
def get_kadastrale_afdeling_by_id(request):
    Gateway = request.capakey_gateway()
    afdeling_id = request.matchdict.get('afdeling_id')
    return Gateway.get_kadastrale_afdeling_by_id(afdeling_id)


@view_config(route_name='list_secties_by_afdeling', renderer='listjson', accept='application/json')
def list_secties_by_afdeling(request):
    Gateway = request.capakey_gateway()
    afdeling_id = request.matchdict.get('afdeling_id')
    secties = Gateway.list_secties_by_afdeling(afdeling_id)
    r = range_return(request)
    return secties[r[0]:r[1]]


@view_config(
    route_name='get_sectie_by_id_and_afdeling',
    renderer='itemjson', accept='application/json'
)
def get_sectie_by_id_and_afdeling(request):
    Gateway = request.capakey_gateway()
    afdeling_id = request.matchdict.get('afdeling_id')
    sectie_id = request.matchdict.get('sectie_id')
    return Gateway.get_sectie_by_id_and_afdeling(sectie_id, afdeling_id)


@view_config(route_name='list_percelen_by_sectie', renderer='listjson', accept='application/json')
def list_percelen_by_sectie(request):
    Gateway = request.capakey_gateway()
    sectie_id = request.matchdict.get('sectie_id')
    afdeling_id = request.matchdict.get('afdeling_id')
    sectie = Gateway.get_sectie_by_id_and_afdeling(sectie_id, afdeling_id)
    percelen = Gateway.list_percelen_by_sectie(sectie)
    r = range_return(request)
    return percelen[r[0]:r[1]]


@view_config(
    route_name='get_perceel_by_sectie_and_id',
    renderer='itemjson', accept='application/json'
)
def get_perceel_by_sectie_and_id(request):
    Gateway = request.capakey_gateway()
    perceel_id = str(request.matchdict.get('perceel_id1'))+'/'\
        + str(request.matchdict.get('perceel_id2'))
    sectie_id = request.matchdict.get('sectie_id')
    afdeling_id = request.matchdict.get('afdeling_id')
    sectie = Gateway.get_sectie_by_id_and_afdeling(sectie_id, afdeling_id)
    return Gateway.get_perceel_by_id_and_sectie(perceel_id, sectie)


@view_config(route_name='get_perceel_by_capakey', renderer='itemjson', accept='application/json')
def get_perceel_by_capakey(request):
    Gateway = request.capakey_gateway()
    capakey = str(request.matchdict.get('capakey1'))+'/'\
        + str(request.matchdict.get('capakey2'))
    return Gateway.get_perceel_by_capakey(capakey)


@view_config(
    route_name='get_perceel_by_percid',
    renderer='itemjson', accept='application/json'
)
def get_perceel_by_percid(request):
    Gateway = request.capakey_gateway()
    percid = request.matchdict.get('percid')
    return Gateway.get_perceel_by_percid(percid)

''' Views for crab services '''

@view_config(route_name='list_gewesten', renderer='listjson', accept='application/json')
def list_gewesten(request):
    Gateway = request.crab_gateway()
    gewesten = Gateway.list_gewesten()
    r = range_return(request)
    return gewesten[r[0]: r[1]]

@view_config(route_name='get_gewest_by_id', renderer='itemjson', accept='application/json')
def get_gewest_by_id(request):
    Gateway = request.crab_gateway()
    gewest_id = int(request.matchdict.get('gewest_id'))
    return Gateway.get_gewest_by_id(gewest_id)
    
@view_config(route_name='list_gemeenten_crab', renderer='listjson', accept='application/json')
def list_gemeenten_crab(request):
    Gateway = request.crab_gateway()
    gewest_id = request.matchdict.get('gewest_id')
    gemeenten = Gateway.list_gemeenten(gewest_id)
    r = range_return(request)
    return gemeenten[r[0]: r[1]]

@view_config(route_name='get_gemeente_crab', renderer='itemjson', accept='application/json')
def get_gemeente_crab(request):
    Gateway = request.crab_gateway()
    gemeente_id = request.matchdict.get('gemeente_id')
    if len(gemeente_id) == 5:
        return Gateway.get_gemeente_by_niscode(gemeente_id)
    else:
        return Gateway.get_gemeente_by_id(gemeente_id)
    

@view_config(route_name='list_straten', renderer='listjson', accept='application/json')
def list_straten(request):
    Gateway = request.crab_gateway()
    gemeente_id = request.matchdict.get('gemeente_id')
    if len(gemeente_id)==5:
        gemeente_id = Gateway.get_gemeente_by_niscode(gemeente_id)
    straten = Gateway.list_straten(gemeente_id)
    r = range_return(request)
    return straten[r[0]: r[1]]
    

@view_config(route_name='get_straat_by_id', renderer='itemjson', accept='application/json')
def get_straat_by_id(request):
    Gateway = request.crab_gateway()
    straat_id = request.matchdict.get('straat_id')
    return Gateway.get_straat_by_id(straat_id)

@view_config(route_name='list_huisnummers', renderer='listjson', accept='application/json')
def list_huisnummers(request):
    Gateway = request.crab_gateway()
    straat_id = request.matchdict.get('straat_id')
    huisnummers = Gateway.list_huisnummers_by_straat(straat_id)
    r = range_return(request)
    return huisnummers[r[0]: r[1]]

@view_config(route_name='get_huisnummer_by_straat_and_label', renderer='itemjson', accept='application/json')
def get_huisnummer_by_straat_and_label(request):
    Gateway = request.crab_gateway()
    straat_id = request.matchdict.get('straat_id')
    huisnummer = request.matchdict.get('huisnummer_label')
    return Gateway.get_huisnummer_by_nummer_and_straat(huisnummer, straat_id)
    
    
@view_config(route_name='get_huisnummer_by_id', renderer='itemjson', accept='application/json')
def get_huisnummer_by_id(request):
    Gateway = request.crab_gateway()
    huisnummer_id = request.matchdict.get('huisnummer_id')
    return Gateway.get_huisnummer_by_id(huisnummer_id)

@view_config(route_name='list_percelen', renderer='listjson', accept='application/json')
def list_percelen(request):
    Gateway = request.crab_gateway()
    huisnummer_id = request.matchdict.get('huisnummer_id')
    percelen = Gateway.list_percelen_by_huisnummer(huisnummer_id)
    r = range_return(request)
    return percelen[r[0]: r[1]]

@view_config(route_name='get_perceel_by_id', renderer='itemjson', accept='application/json')
def get_perceel_by_id(request):
    Gateway = request.crab_gateway()
    perceel_id = request.matchdict.get('perceel_id1')+'/'+request.matchdict.get('perceel_id2')
    return Gateway.get_perceel_by_id(perceel_id)

@view_config(route_name='list_gebouwen', renderer='listjson', accept='application/json')
def list_gebouwen(request):
    Gateway = request.crab_gateway()
    huisnummer_id = request.matchdict.get('huisnummer_id')
    gebouwen = Gateway.list_gebouwen_by_huisnummer(huisnummer_id)
    r = range_return(request)
    return gebouwen[r[0]: r[1]]

@view_config(route_name='get_gebouw_by_id', renderer='itemjson', accept='application/json')
def get_gebouw_by_id(request):
    Gateway = request.crab_gateway()
    gebouw_id = request.matchdict.get('gebouw_id')
    return Gateway.get_gebouw_by_id(gebouw_id)
